/*
 * jFinisher is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    jFinisher is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package br.ufpr.tads.jfinisher;

import br.ufpr.tads.jfinisher.frame.MainFrame;
import br.ufpr.tads.jfinisher.util.TextTransfer;
import java.awt.Font;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ResourceBundle;

public class JFinisher {

    //Tela principal
    public static MainFrame mainFrame;
    //Lista de sequencias abertas
    private static ArrayList<OpenSequence> openSequences = new ArrayList<OpenSequence>();
    //public static OptionsFrame optionsFrame;
    /* Variaveis Globais */
    //Padres de fonte (valores carregados na inicializao)
    public static int fontWidht;
    public static int fontHeight;
    //Titulo
    private static final String titleBase = "jFinisher";
    //numero de caracteres por label (max ~3273)
    public static final int labelBase = 3000;
    //Recurso de texto - multi-lingua
    public static final ResourceBundle textResource = ResourceBundle.getBundle("br/ufpr/tads/jfinisher/resource/messages");
    //Area de transferencia
    public static TextTransfer textTransfer = new TextTransfer();
    /* Propriedades */
    public static final Font font = new Font("Monospaced", 0, 13);
    //Frequencia do contador
    public static final int basesCont = 20;
    //Tamanho do scroll dos paineis
    public static final int scrollUnitIncrement = 16;
    //Tamanho da descricao
    public static final int descriptionSize = 20;
    //Caractere que preenche espao vazio em sequencia
    public static final byte fillChar = "N".getBytes()[0];
    //Valores temporarios alinhamento
    public static int tmpSeedLenght = 15;
    public static float tmpOpenGapPenalty = 10.0f;
    public static float tmpExtendGapPenalty = 0.5f;
    //Projeto aberto
    private static String openProject = null;

    public static void main(String args[]) {
        mainFrame = new MainFrame();
        //optionsFrame = new OptionsFrame();

        //Utiliza componente da tela para pegar tamanho padrao das fonte
        JFinisher.fontWidht = mainFrame.getFontMetrics(font).stringWidth("*");
        JFinisher.fontHeight = mainFrame.getFontMetrics(font).getHeight();

        mainFrame.setTitle(titleBase);
        mainFrame.setSize(800, 600);
        mainFrame.setLocationRelativeTo(null);
        mainFrame.setVisible(true);

    }

    public static String getOpenProject() {
        return openProject;
    }

    public static void setOpenProject(String openProject) {
        JFinisher.openProject = openProject;
        if (openProject == null) {
            setTitle(null);
        } else {
            setTitle(openProject.substring(openProject.lastIndexOf("/") + 1, openProject.length()));
        }
    }

    public static void setTitle(String title) {
        if (title == null) {
            mainFrame.setTitle(titleBase);
        } else {
            mainFrame.setTitle(titleBase + " - " + title);
        }
    }

    public static boolean isFirstAlignment() {
        for (OpenSequence os : getOpenSequences()) {
            if (os.getAlignment().size() > 0) {
                return false;
            }
        }
        return true;
    }

    public static Sequence createConsensus(List<OpenSequence> openSequences) {
        int posI = -1, posF = -1, auxI, auxF;
        for (OpenSequence os : openSequences) {
            auxI = os.getPosition();
            auxF = os.getPosition() + os.getSequence().getLength() - 1;
            if (auxI < posI || posI == -1) {
                posI = auxI;
            }
            if (auxF > posF || posF == -1) {
                posF = auxF;
            }
        }
        return createConsensus(openSequences, posI, posF);
    }

    public static Sequence createConsensus(List<OpenSequence> openSequences, int ini, int end) {

        System.out.println("Generating consensus (" + ini + " to " + end + ")...");
        long begintime = System.currentTimeMillis();

        //Armazena valores das colunas de cada posicao de cada contig aberto
        StringBuilder columnValues = null;

        //Armazena nova sequencia formada
        StringBuilder newSequence = new StringBuilder();

        //Loop nos contigs (linha)
        for (int c = 0; c <= end - ini; c++) {
            //Posio atual analisada
            int posA = ini + c;

            HashMap<Character, int[]> hm = new HashMap<Character, int[]>();

            //Loop nas sequencias (contigs) abertas (coluna)
            for (OpenSequence os : openSequences) {

                //So concatena sequencias que estiverem na coluna analisada
                int posI = 0, posF = 0;
                boolean range = false;

                //Posicao inicial do contig (para verificacao se esta no range para entrar no consenso)
                posI = os.getPosition();
                //Posicao final do contig
                posF = os.getPosition() + os.getSequence().getLength() - 1;

                //Caso esteja no range 
                if (posA >= posI && posA <= posF) {
                    range = true;
                }

                //Caso esteja no range analisado
                if (range || (posA >= posI && posA <= posF)) {
                    //Define qual posicao do contig analisado na posicao atual geral
                    int posContig = ini - posI + c;

                    //Adiciona valor ao hashmap
                    char currentChar = os.getSequence().getSeqStr(posContig, posContig).charAt(0);
                    if (hm.get(currentChar) != null) {
                        //hm.put(cc, (hm.get(cc)) + 1);

                        //Verifica se sequencia analisada  maior que a definida no hash (para desempate)
                        int longestSequence;
                        if (hm.get(currentChar)[1] < os.getSequence().getLength()) {
                            longestSequence = os.getSequence().getLength();
                        } else {
                            longestSequence = hm.get(currentChar)[1];
                        }

                        int val[] = {hm.get(currentChar)[0] + 1, longestSequence};
                        hm.put(currentChar, val);

                    } else {
                        //Primeira ocorrencia do caractere
                        int t[] = {1, os.getSequence().getLength()};
                        hm.put(currentChar, t);
                    }

                }
            }

            //Caso avaliacao retorne vazia (posicao sem contigs) preenche com fill
            if (hm.isEmpty()) {
                newSequence.append((char) JFinisher.fillChar);
            } else {

                int maxCount = 0, maxLength = 0;
                String newChar = null;
                //Loop no hashmap para pegar maior valor (frequencia de base)
                for (Character key : hm.keySet()) {

                    //Caso tenha contador igual
                    if (hm.get(key)[0] == maxCount) {
                        //Desempata pela maior sequencia
                        if (hm.get(key)[1] > maxLength) {
                            maxCount = hm.get(key)[0];
                            maxLength = hm.get(key)[1];
                            newChar = key.toString();
                        }
                    } else if (hm.get(key)[0] > maxCount) {
                        //Caso seja maior que atual, sobrepoe
                        maxCount = hm.get(key)[0];
                        maxLength = hm.get(key)[1];
                        newChar = key.toString();
                    }

                }
                //Gera nova sequencia
                newSequence.append(newChar.toUpperCase());

            }
        }

        long endtime = System.currentTimeMillis();
        System.out.println("Time elapsed: " + (endtime - begintime) + "ms");
        return new Sequence(newSequence.toString().getBytes());
    }

    public static OpenSequence getOpenSequence(int index) {
        return openSequences.get(index);
    }

    public static ArrayList<OpenSequence> getOpenSequences() {
        return openSequences;
    }

    public static int getOpenSequencesSize() {
        return openSequences.size();
    }

    public static int getOpenSequenceIndex(OpenSequence openSequence) {
        return openSequences.indexOf(openSequence);
    }

    public static void addOpenSequence(OpenSequence openSequence) {
        openSequences.add(openSequence);
    }

    public static void setOpenSequences(ArrayList<OpenSequence> openSequences) {
        JFinisher.openSequences = openSequences;
    }

    public static void removeOpenSequence(OpenSequence openSequence) {
        openSequences.remove(openSequence);
    }

    public static void clearOpenSequences() {
        openSequences = new ArrayList<OpenSequence>();
    }

    public static List<OpenSequence> getContigsOpenSequence() {
        return openSequences.subList(1, openSequences.size());
    }

    public static OpenSequence getReferenceOpenSequence() {
        if (openSequences.size() > 0) {
            return openSequences.get(0);
        }
        return null;
    }
}
