package br.ufpr.tads.jfinisher;


import br.ufpr.bioinfo.genbank.Entry;
import br.ufpr.bioinfo.sequence.fasta.Fasta;
import br.ufpr.bioinfo.sequence.fasta.FastaUtil;
import br.ufpr.tads.jfinisher.sequencepanel.SequencePanelSeq;
import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JLabel;

public class OpenSequence {

    //Nome do arquivo
    private String file;
    //Descrio padro
    private String description;
    //Sequencia
    private Sequence sequence;
    //Painel que contem esta sequencia
    private SequencePanelSeq sequencePanel;
    //Label que faz referencia no mapa
    private JLabel mapLabel;
    //Alinhamentos da sequencia
    private List<OpenSequenceAlignment> alignment = new ArrayList<OpenSequenceAlignment>();
    //Componente que alinha a sequencia
    private Component aligner;
    //Parte selecionada dentro da sequencia
    private SelectedSequence selectedSeq;
    //Verifica se sequencia faz parte de arquivo de multiplas sequencias ou unico
    private boolean multiSequenceFile = false;
    //Entradas arquivos .gbk
    private Entry gbkEntry;
    //Verifica se  referencia
    private boolean referenceSequence;
    private String sequenceHash;

    public OpenSequence() {
        //Instancia a sequencia selecionada (vazia)
        selectedSeq = new SelectedSequence();
    }

    public String getFile() {
        return file;
    }

    public void setFile(String file) {
        this.file = file;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public SequencePanelSeq getSequencePanel() {
        return sequencePanel;
    }

    public void setSequencePanel(SequencePanelSeq sequencePanel) {
        this.sequencePanel = sequencePanel;
    }

    public List<OpenSequenceAlignment> getAlignment() {
        return alignment;
    }

    public void setAlignment(List<OpenSequenceAlignment> alignment) {
        this.alignment = alignment;
    }

    public void addAlignment(OpenSequenceAlignment alignment) {
        this.alignment.add(alignment);
    }

    public Sequence getSequence() {
        return sequence;
    }

    public void setSequence(Sequence seq) {
        this.sequence = seq;
        this.setSequenceHash(seq.getSeq());
    }

    public class SelectedSequence {

        private int start;
        private int end;

        public SelectedSequence() {
            this.start = -1;
            this.end = -1;
        }

        public void setBounds(int st, int en) {
            if (st == -1 || en == -1) {
                this.clear();
            } else {
                if (st > en) {
                    this.setStart(en);
                    this.setEnd(st);
                } else {
                    this.setStart(st);
                    this.setEnd(en);
                }
            }
        }

        public int getEnd() {
            return end;
        }

        public void setEnd(int en) {
            this.end = en;
        }

        public Sequence getSeq() {
            Sequence seq = new Sequence(getSequence().getSeq(getStart(), getEnd()));
            return seq;
        }

        public int getStart() {
            return start;
        }

        public void setStart(int st) {
            if (st < 0) {
                st = 0;
            }
            this.start = st;
        }

        public void clear() {
            this.start = -1;
            this.end = -1;
        }
    }

    public SelectedSequence getSelectedSeq() {
        return selectedSeq;
    }

    public void setSelectedSeq(SelectedSequence selectedSeq) {
        this.selectedSeq = selectedSeq;
    }

    public Component getAligner() {
        return aligner;
    }

    public void setAligner(Component aligner) {
        this.aligner = aligner;
    }

    public void setAlignerPosition(int charPos) {
        int x = charPos * JFinisher.fontWidht;
        this.aligner.setPreferredSize(new Dimension(x, 0));
    }

    public int getPosition(){
        if(this.getOpenSequenceAlignmentSelected()!=null){
            return this.getOpenSequenceAlignmentSelected().getPosition();
        }else{
            return 0;
        }
    }
    public Entry getGbkEntry() {
        return gbkEntry;
    }

    public void setGbkEntry(Entry gbkEntry) {
        this.gbkEntry = gbkEntry;
    }

    public boolean isMultiSequenceFile() {
        return multiSequenceFile;
    }

    public void setMultiSequenceFile(boolean multiSequenceFile) {
        this.multiSequenceFile = multiSequenceFile;
    }

    public boolean isReferenceSequence() {
        return referenceSequence;
    }

    public void setReferenceSequence(boolean referenceSequence) {
        this.referenceSequence = referenceSequence;
    }

    public JLabel getMapLabel() {
        return mapLabel;
    }

    public void setMapLabel(JLabel mapLabel) {
        this.mapLabel = mapLabel;
    }

    public void setSequenceHash(byte[] sequenceHash) {
        try {
            MessageDigest md = MessageDigest.getInstance("SHA-1");
            md.update(sequenceHash);
            this.sequenceHash = stringHex(md.digest());
        } catch (NoSuchAlgorithmException e) {
        }
    }

    public String getSequenceHash() {
        return this.sequenceHash;
    }
    
    public void removeAllOpenSequenceAlignment(){
        this.getAlignment().removeAll(alignment);
    }
    public void removeOpenSequenceAlignment(OpenSequenceAlignment osa) {
        this.getAlignment().remove(osa);
    }

    private static String stringHex(byte[] bytes) {
        StringBuilder s = new StringBuilder();
        for (int i = 0; i < bytes.length; i++) {
            int parteAlta = ((bytes[i] >> 4) & 0xf) << 4;
            int parteBaixa = bytes[i] & 0xf;
            if (parteAlta == 0) {
                s.append('0');
            }
            s.append(Integer.toHexString(parteAlta | parteBaixa));
        }
        return s.toString();
    }

    public void deleteSelectedSequence(SelectedSequence ss) {

        //Monta nova sequencia
        Sequence sequencePart1 = new Sequence(this.getSequence().getSeq(0, ss.getStart() - 1));
        Sequence sequencePart2 = new Sequence(this.getSequence().getSeq(ss.getEnd() + 1, this.getSequence().getLength() - 1));
        sequencePart1.appendSeq(sequencePart2);

        //Atualiza sequencia
        this.setSequence(sequencePart1);

        //Atualiza arquivo
        writeFile();
    }

    public void insertSequence(String seq, SelectedSequence ss) {

        //Monta nova sequencia
        Sequence sequencePart1 = new Sequence(this.getSequence().getSeq(0, ss.getStart() - 1));
        Sequence sequencePart2 = new Sequence(seq);
        Sequence sequencePart3 = new Sequence(this.getSequence().getSeq(ss.getStart(), this.getSequence().getLength() - 1));
        sequencePart1.appendSeq(sequencePart2);
        sequencePart1.appendSeq(sequencePart3);


        sequencePart1.getSeqStr();
        sequencePart2.getSeqStr();
        sequencePart3.getSeqStr();

        //Atualiza sequencia
        this.setSequence(sequencePart1);

        //Atualiza arquivo
        writeFile();

    }

    public void rewriteSequence(Sequence s) {
        //Atualiza sequencia
        this.setSequence(s);
        //Atualiza arquivo
        writeFile();
    }

    private void writeFile() {

        if (this.getFile().endsWith(".gbk")) {
        } else {
            //Cria lista com sequencias restantes do arquivo (menos a excluida)
            ArrayList<Fasta> fastas = new ArrayList<Fasta>();
            //Percorre todas as sequencias abertas
            for (OpenSequence auxOs : JFinisher.getContigsOpenSequence()) {
                //Caso seja do mesmo arquivo
                if (auxOs.getFile().equals(this.getFile())) {
                    Fasta fst = new Fasta();
                    fst.setSequence(auxOs.getSequence().getSeq());
                    fst.setHeader(auxOs.getDescription());
                    fastas.add(fst);
                }
            }
            //Reescreve arquivo
            FastaUtil.writeFasta(fastas, new File(this.getFile()));
        }
    }

    public void removeFromFile() {
        //Cria lista com sequencias restantes do arquivo (menos a excluida)
        ArrayList<Fasta> fastas = new ArrayList<Fasta>();

        //Percorre todas as sequencias abertas
        for (OpenSequence auxOs : JFinisher.getContigsOpenSequence()) {
            //Caso seja do mesmo arquivo
            if (auxOs.getFile().equals(this.getFile())) {
                //Todas menos a que est sendo excluida
                if (auxOs != this) {
                    Fasta fst = new Fasta();
                    fst.setSequence(auxOs.getSequence().getSeq());
                    fst.setHeader(auxOs.getDescription());
                    fastas.add(fst);
                }
            }
        }

        //Reescreve arquivo
        FastaUtil.writeFasta(fastas, new File(this.getFile()));
    }

    public OpenSequenceAlignment getOpenSequenceAlignmentSelected() {
        for (OpenSequenceAlignment auxOsa : this.getAlignment()) {
            if (auxOsa.isSelected()) {
                return auxOsa;
            }
        }
        return null;
    }

    public void setOpenSequenceAlignmentSelected(OpenSequenceAlignment osa) {
        //Seta todos como falso
        for (OpenSequenceAlignment auxOsa : this.getAlignment()) {
            auxOsa.setSelected(false);
        }
        if (osa != null) {
            osa.setSelected(true);
        }
    }

    public int searchSequence(String search) {
        return this.getSequence().getSeqStr().indexOf(search);
    }

    public Sequence getSequenceWithoutAlignmentArea(OpenSequence openSequenceWithAlignment) {
        if (this.isReferenceSequence()) {
            //Elimina da referencia sequencia que ja estao alinhadas (preenche com caractere nulo)
            //Preenche area alinhada na referencia com N para nao alinhar novamente na mesma posicao
            Sequence s = new Sequence(this.getSequence().getSeq());
            for (int a = 0; a < openSequenceWithAlignment.getAlignment().size(); a++) {
                s.setSeqFill(openSequenceWithAlignment.getAlignment().get(a).getPosition(), openSequenceWithAlignment.getAlignment().get(a).getPosition() + openSequenceWithAlignment.getSequence().getLength(), JFinisher.fillChar);
            }
            return s;
        }
        return null;
    }
}
