/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package br.ufpr.tads.jfinisher.action;

import br.ufpr.bioinfo.sequence.fasta.Fasta;
import br.ufpr.bioinfo.sequence.fasta.FastaUtil;
import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.frame.MainFrame;
import br.ufpr.tads.jfinisher.OpenSequence;
import br.ufpr.tads.jfinisher.enumerator.ExportType;
import br.ufpr.tads.jfinisher.util.WindowUtil;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 *
 * @author vitor
 */
public class ExportAction extends AbstractAction implements Action {

    private FileNameExtensionFilter fe = null;
    private ExportType exportType;

    public ExportAction(MainFrame mf, ExportType et) {
        this.exportType = et;

        if (exportType == ExportType.MAP) {
            fe = new FileNameExtensionFilter(JFinisher.textResource.getString("ExportAction.exportImage.files"), "png");
        } else {
            fe = new FileNameExtensionFilter(JFinisher.textResource.getString("ExportAction.exportSequence.files"), "fas", "fa", "fasta", "fna", "ffn", "faa");
        }
    }

    @Override
    public void actionPerformed(ActionEvent e) {

        Thread t = new Thread() {

            @Override
            public void run() {
                boolean doWrite = false;
                JFileChooser openDialog = WindowUtil.getOpenChooser();
                openDialog.setDialogTitle(JFinisher.textResource.getString("ExportAction.title"));
                openDialog.setDialogType(JFileChooser.SAVE_DIALOG);
                openDialog.setFileFilter(fe);
                openDialog.setCurrentDirectory(new File("."));
                if (exportType == ExportType.MAP) {
                    openDialog.setSelectedFile(new File("*.png"));
                } else if (exportType == ExportType.CONSENSUS) {
                    openDialog.setSelectedFile(new File("*.fasta"));
                } else if (exportType == ExportType.SEQUENCES) {
                    openDialog.setSelectedFile(new File("*.ffn"));
                }

                if (openDialog.showSaveDialog(JFinisher.mainFrame) == JFileChooser.APPROVE_OPTION) {
                    if (openDialog.getSelectedFile().exists()) {
                        int overwrite = JOptionPane.showConfirmDialog(JFinisher.mainFrame, JFinisher.textResource.getString("SaveProjectAction.fileExists"), JFinisher.textResource.getString("SaveProjectAction.title") + " [" + openDialog.getSelectedFile().getName() + "]", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
                        if (overwrite == JOptionPane.YES_OPTION) {
                            doWrite = true;
                        }
                    } else {
                        doWrite = true;
                    }
                }
                if (doWrite) {
                    try {

                        if (exportType == ExportType.MAP) {
                            exportMap(openDialog.getSelectedFile().getAbsoluteFile());
                        } else if (exportType == ExportType.CONSENSUS) {
                            exportConsensus(openDialog.getSelectedFile().getAbsoluteFile());
                        } else if (exportType == ExportType.SEQUENCES) {
                            exportSequences(openDialog.getSelectedFile().getAbsoluteFile());
                        }
                        JOptionPane.showMessageDialog(JFinisher.mainFrame, JFinisher.textResource.getString("Message.fileGenerated") + ": " + openDialog.getSelectedFile().getName(), JFinisher.textResource.getString("Message.title"), JOptionPane.INFORMATION_MESSAGE);
                    } catch (IOException ex) {
                        JOptionPane.showMessageDialog(JFinisher.mainFrame, JFinisher.textResource.getString("Error.unknown"), JFinisher.textResource.getString("Error.title"), JOptionPane.ERROR_MESSAGE);
                    } catch (OutOfMemoryError ex) {
                        JOptionPane.showMessageDialog(JFinisher.mainFrame, JFinisher.textResource.getString("Error.memory"), JFinisher.textResource.getString("Error.title"), JOptionPane.ERROR_MESSAGE);
                    } catch (NegativeArraySizeException ex) {
                        JOptionPane.showMessageDialog(JFinisher.mainFrame, JFinisher.textResource.getString("Error.unknown"), JFinisher.textResource.getString("Error.title"), JOptionPane.ERROR_MESSAGE);
                    }
                }
                openDialog.removeChoosableFileFilter(fe);
            }
        };
        t.start();
    }

    private void exportMap(File fl) throws IOException, OutOfMemoryError, NegativeArraySizeException {
        BufferedImage bi = JFinisher.mainFrame.sspMap.createImage(); // retrieve image
        ImageIO.write(bi, "png", fl);
    }

    private void exportConsensus(File fl) throws IOException, OutOfMemoryError, NegativeArraySizeException {
        Fasta fasta = new Fasta();
        fasta.setHeader("Sequencia consenso");
        fasta.setSequence(JFinisher.mainFrame.sspReference.getConsensusSequence().getSeq());
        //Reescreve arquivo
        FastaUtil fstUtil = new FastaUtil();
        fstUtil.writeFasta(fasta, fl);
    }

    private void exportSequences(File fl) throws IOException, OutOfMemoryError, NegativeArraySizeException {
        ArrayList<Fasta> fastas = new ArrayList<Fasta>();
        for (OpenSequence auxOs : JFinisher.getContigsOpenSequence()) {
            Fasta fst = new Fasta();
            fst.setSequence(auxOs.getSequence().getSeq());
            fst.setHeader(auxOs.getDescription());
            fastas.add(fst);
        }
        FastaUtil fstUtil = new FastaUtil();
        fstUtil.writeFasta(fastas, fl);
    }
}
