package br.ufpr.tads.jfinisher.action;

import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.OpenSequence;
import br.ufpr.tads.jfinisher.OpenSequenceAlignment;
import br.ufpr.tads.jfinisher.util.WindowUtil;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.StaxDriver;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

public class LoadProjectAction extends AbstractAction implements Action {

    private FileNameExtensionFilter fe = null;
    OpenAction openAction = new OpenAction();

    public LoadProjectAction() {
        fe = new FileNameExtensionFilter(JFinisher.textResource.getString("LoadProjectAction.files"), "xml");
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Thread t = new Thread() {

            @Override
            public void run() {
                JFileChooser openDialog = WindowUtil.getOpenChooser();
                openDialog.setDialogTitle(JFinisher.textResource.getString("LoadProjectAction.title"));
                openDialog.setDialogType(JFileChooser.OPEN_DIALOG);
                openDialog.setFileFilter(fe);
                if (openDialog.showOpenDialog(JFinisher.mainFrame) == JFileChooser.APPROVE_OPTION) {
                    loadProject(openDialog.getSelectedFile());
                }
                openDialog.removeChoosableFileFilter(fe);
            }
        };
        t.start();
    }

    private void loadProject(File projectFile) {

        //Caso existam sequencias abertas, avisa que alteracoes serao perdidas
        int unsavedChanges = JOptionPane.YES_OPTION;
        if (!JFinisher.getOpenSequences().isEmpty()) {
            unsavedChanges = JOptionPane.showConfirmDialog(JFinisher.mainFrame, JFinisher.textResource.getString("LoadProjectAction.unsavedChanges"), JFinisher.textResource.getString("LoadProjectAction.title"), JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);
        }

        if (unsavedChanges == JOptionPane.YES_OPTION) {

            //Limpa tela, arquivos abertos, etc
            JFinisher.mainFrame.clear();

            //Instancia do xstrem
            XStream xstream = new XStream(new StaxDriver());

            //Loop nas sequencias do arquivo xml
            for (OpenSequence fileOpenSequences : (ArrayList<OpenSequence>) xstream.fromXML(projectFile)) {

                //Arquivo da sequencia extraido do xml
                File sequenceFile = new File(fileOpenSequences.getFile());

                //Tenta abrir, caso no encontre arquivo da erro, catch
                try {
                    //Carrega arquivo salvo no xml (somente abre se descricao e sequencia nao forem alteradas)
                    OpenSequence loadedOpenSequence = openAction.openSpecificFile(sequenceFile, fileOpenSequences.getDescription(), fileOpenSequences.getSequenceHash());
                    if (loadedOpenSequence != null) {
                        addSequence(loadedOpenSequence, fileOpenSequences.getAlignment());
                    } else {
                        //Caso no encontre a sequencia, joga para recuperacao
                        throw new IOException();
                    }
                } catch (IOException ex) {

                    //Pergunta se quer procurar arquivo
                    int findFile = JOptionPane.showConfirmDialog(JFinisher.mainFrame, fileOpenSequences.getFile() + "\n" + fileOpenSequences.getDescription() + "\n\n" + JFinisher.textResource.getString("LoadProjectAction.findFile"), JFinisher.textResource.getString("LoadProjectAction.title"), JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);

                    //Caso no deseje encontrar aquivo e seja referencia, limpa tela e sai
                    if (findFile == JOptionPane.NO_OPTION && fileOpenSequences.isReferenceSequence()) {
                        JFinisher.mainFrame.clear();
                        return;
                    } else {
                        //Caso queira procurar
                        if (findFile == JOptionPane.YES_OPTION) {

                            //Procura arquivo
                            OpenSequence newFindFile = findFile(fileOpenSequences);

                            //Se encontrou
                            if (newFindFile != null) {
                                addSequence(newFindFile, fileOpenSequences.getAlignment());
                            } else if (fileOpenSequences.isReferenceSequence()) {
                                //Caso desista de encontar arquivo e seja referencia, sai
                                JFinisher.mainFrame.clear();
                                return;
                            }
                        }
                    }
                }

                //Caso seja a primeira, atualiza informaes de referencia
                if (fileOpenSequences.isReferenceSequence()) {
                    JFinisher.mainFrame.posOpenReference();
                }
            }

            //Atualiza informaes dos contigs
            JFinisher.mainFrame.posOpenContig();

            //Define projeto atual
            JFinisher.setOpenProject(projectFile.getAbsolutePath());

        }
    }

    private OpenSequence findFile(OpenSequence fileOpenSequences) {
        boolean find = false, wrongFile = false;
        OpenSequence loadedOpenSequence = null;

        //Abre dialogo correto conforme arquivo
        JFileChooser openDialog;
        if (fileOpenSequences.isReferenceSequence()) {
            OpenReferenceAction opa = new OpenReferenceAction();
            openDialog = opa.openDialog();
        } else {
            OpenContigsAction opa = new OpenContigsAction();
            openDialog = opa.openDialog(false);
        }

        //loop at encontrar arquivo ou desistir
        while (!find) {
            if (openDialog.showOpenDialog(JFinisher.mainFrame) == JFileChooser.APPROVE_OPTION) {
                try {
                    //Carrega arquivo salvo no xml (comparando apenas hash)
                    loadedOpenSequence = openAction.openSpecificFile(openDialog.getSelectedFile(), fileOpenSequences.getSequenceHash());
                    //Caso possuam a mesma sequencia, deixa passar
                    if (loadedOpenSequence != null) {
                        find = true;
                    } else {
                        wrongFile = true;
                        find = false;
                    }
                } catch (IOException ex) {
                    JOptionPane.showMessageDialog(JFinisher.mainFrame, JFinisher.textResource.getString("Error.openFile"), JFinisher.textResource.getString("Error.title"), JOptionPane.ERROR_MESSAGE);
                }

            } else {
                wrongFile = true;
                find = false;
            }

            //Caso arquivo esteja incorreto ou cancelou, pergunta novamente
            if (wrongFile == true) {
                if (JOptionPane.showConfirmDialog(JFinisher.mainFrame, fileOpenSequences.getFile() + "\n" + fileOpenSequences.getDescription() + "\n\n" + JFinisher.textResource.getString("LoadProjectAction.wrongFile"), JFinisher.textResource.getString("LoadProjectAction.title"), JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.NO_OPTION) {
                    find = true;
                }
            }
        }
        return loadedOpenSequence;
    }

    private void addSequence(OpenSequence loadedOpenSequence, List<OpenSequenceAlignment> osa_) {
        openAction.addSequence(loadedOpenSequence);
        //Carrega alinhamentos
        for (OpenSequenceAlignment osa : osa_) {
            loadedOpenSequence.addAlignment(osa);
            JFinisher.mainFrame.sspMap.addSequenceAlignment(loadedOpenSequence, osa);
        }
        if (!loadedOpenSequence.isReferenceSequence() && loadedOpenSequence.getAlignment().size() > 0) {
            OpenSequenceAlignment lastOsa = loadedOpenSequence.getAlignment().get(loadedOpenSequence.getAlignment().size() - 1);
            //Marca sequencia no mapa
            JFinisher.mainFrame.sspMap.paintAlign(loadedOpenSequence, lastOsa);
            //Move sequencia na tela
            JFinisher.mainFrame.sspContigs.moveSequence(loadedOpenSequence, lastOsa, null);
        }
    }
}