package br.ufpr.tads.jfinisher.action;

import br.ufpr.bioinfo.genbank.Entry;
import br.ufpr.bioinfo.genbank.parser.GenBankParser;
import br.ufpr.bioinfo.sequence.fasta.Fasta;
import br.ufpr.bioinfo.sequence.fasta.FastaUtil;
import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.frame.MainFrame;
import br.ufpr.tads.jfinisher.OpenSequence;
import br.ufpr.tads.jfinisher.Sequence;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

public class OpenAction extends AbstractAction implements Action {

    public ArrayList<OpenSequence> openMultiFasta(File selectedFile) throws IOException {
        if (selectedFile.exists() == false) {
            throw new IOException();
        }
        OpenSequence os;
        ArrayList<OpenSequence> oss = new ArrayList<OpenSequence>();
        List<Fasta> fastas = new ArrayList<Fasta>();
        fastas = FastaUtil.read(selectedFile);
        for (int i = 0; i < fastas.size(); i++) {
            os = new OpenSequence();
            os.setFile(selectedFile.getAbsolutePath());
            os.setDescription(fastas.get(i).getHeader());
            os.setSequence(new Sequence(fastas.get(i).getSequence()));
            if (fastas.size() > 1) {
                os.setMultiSequenceFile(true);
            }
            if (JFinisher.getOpenSequences().isEmpty()) {
                os.setReferenceSequence(true);
            }
            oss.add(os);
        }
        return oss;
    }

    public OpenSequence openFasta(File selectedFile) throws IOException {
        if (selectedFile.exists() == false) {
            throw new IOException();
        }
        OpenSequence os = new OpenSequence();
        Fasta fas = FastaUtil.readFirstSequence(selectedFile.getAbsolutePath());
        os.setFile(selectedFile.getAbsolutePath());
        os.setDescription(fas.getHeader());
        os.setSequence(new Sequence(fas.getSequence()));
        if (JFinisher.getOpenSequences().isEmpty()) {
            os.setReferenceSequence(true);
        }
        return os;
    }

    public OpenSequence openGenBank(File selectedFile) throws IOException {
        if (selectedFile.exists() == false) {
            throw new IOException();
        }
        OpenSequence os = new OpenSequence();
        GenBankParser gp = new GenBankParser();
        Entry entry = gp.processFile(selectedFile);
        os.setFile(selectedFile.getAbsolutePath());
        if (entry.getLocus() != null) {
            os.setDescription(entry.getLocus().getName());
        } else {
            os.setDescription("GBK FILE");
        }
        String seq = entry.getSequence().getSequence();
        if (seq.equals("")) {
            throw new IOException();
        } else {
            os.setSequence(new Sequence(seq));
            if (JFinisher.getOpenSequences().isEmpty()) {
                os.setReferenceSequence(true);
            }
            os.setGbkEntry(entry);
            return os;
        }
    }

    public void addSequence(ArrayList<OpenSequence> os) {
        for (OpenSequence openSequence : os) {
            addSequence(openSequence);
        }
    }

    public void addSequence(OpenSequence os) {
        boolean error = false;
        if (!JFinisher.getOpenSequences().isEmpty()) {
            if (os.getSequence().getLength() > JFinisher.getReferenceOpenSequence().getSequence().getLength()) {
                JOptionPane.showMessageDialog(JFinisher.mainFrame, JFinisher.textResource.getString("Open.widerContig") + "\n" + os.getDescription());
                error = true;
            }
        }

        if (error == false) {
            
            long begin = System.currentTimeMillis();

            //Adiciona sequencia
            JFinisher.addOpenSequence(os);
            
            //Primeira sequencia (referencia)
            if (os.isReferenceSequence()) {
                JFinisher.mainFrame.sspReference.addSequence(os);
                JFinisher.mainFrame.sspContigs.addHorizontalSpace();
            } else {
                JFinisher.mainFrame.sspContigs.addSequence(os);
            }

            JFinisher.mainFrame.sspMap.addSequence(os);

            long end = System.currentTimeMillis();
            System.out.println("Opening sequence " + os.getDescription() + ": " + (end - begin) + "ms");
        }
    }

    public ArrayList<OpenSequence> openFile(File openFile) throws IOException {
        String auxExtension = openFile.getName().toLowerCase();
        ArrayList<OpenSequence> openSequences = new ArrayList<OpenSequence>();
        OpenSequence os = null;
        if (auxExtension.endsWith(".gbk")) {
            os = openGenBank(openFile);
            openSequences.add(os);
        } else if (auxExtension.endsWith(".fas") || auxExtension.endsWith(".fa") || auxExtension.endsWith(".fasta") || auxExtension.endsWith(".fna")) {
            os = openFasta(openFile);
            openSequences.add(os);
        } else if (auxExtension.endsWith(".ffn") || auxExtension.endsWith(".faa")) {
            for (OpenSequence oss : openMultiFasta(openFile)) {
                openSequences.add(oss);
            }
        }
        return openSequences;
    }

    public OpenSequence openSpecificFile(File openFile, String description, String sequenceHash) throws IOException {
        String auxExtension = openFile.getName().toLowerCase();
        OpenSequence os = null;
        if (auxExtension.endsWith(".gbk")) {
            os = openGenBank(openFile);
            if (os.getDescription().equals(description)
                    && os.getSequenceHash().equals(sequenceHash)) {
                return os;
            }
        } else if (auxExtension.endsWith(".fas") || auxExtension.endsWith(".fa") || auxExtension.endsWith(".fasta") || auxExtension.endsWith(".fna")) {
            os = openFasta(openFile);
            if (os.getDescription().equals(description)
                    && os.getSequenceHash().equals(sequenceHash)) {
                return os;
            }
        } else if (auxExtension.endsWith(".ffn") || auxExtension.endsWith(".faa")) {
            for (OpenSequence oss : openMultiFasta(openFile)) {
                if (oss.getDescription().equals(description)
                        && oss.getSequenceHash().equals(sequenceHash)) {
                    return oss;
                }
            }
        }
        return null;
    }

    public OpenSequence openSpecificFile(File openFile, String sequenceHash) throws IOException {
        String auxExtension = openFile.getName().toLowerCase();
        OpenSequence os = null;
        if (auxExtension.endsWith(".gbk")) {
            os = openGenBank(openFile);
            if (os.getSequenceHash().equals(sequenceHash)) {
                return os;
            }
        } else if (auxExtension.endsWith(".fas") || auxExtension.endsWith(".fa") || auxExtension.endsWith(".fasta") || auxExtension.endsWith(".fna")) {
            os = openFasta(openFile);
            if (os.getSequenceHash().equals(sequenceHash)) {
                return os;
            }
        } else if (auxExtension.endsWith(".ffn") || auxExtension.endsWith(".faa")) {
            for (OpenSequence oss : openMultiFasta(openFile)) {
                if (oss.getSequenceHash().equals(sequenceHash)) {
                    return oss;
                }
            }
        }
        return null;
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
    }
}
