package br.ufpr.tads.jfinisher.align;

import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.frame.AlignFrame;
import jaligner.SmithWatermanGotoh;
import jaligner.matrix.Matrix;
import jaligner.matrix.MatrixLoader;
import jaligner.matrix.MatrixLoaderException;
import jaligner.util.SequenceParser;
import br.ufpr.tads.jfinisher.OpenSequenceAlignment;
import br.ufpr.tads.jfinisher.Sequence;
import jaligner.util.SequenceParserException;

public class AlignerJA extends Thread {

    private Sequence contigSequence;
    private Sequence referenceSequence;
    private int openSequenceIndex;
    private float openGapPenalty;
    private float extendGapPenalty;
    private AlignFrame alignFrame;
    private int median = -1;

    public AlignerJA(float openGapPenalty, float extendGapPenalty, Sequence os, Sequence osr, int m, AlignFrame af, int i) {
        this.openGapPenalty = openGapPenalty;
        this.extendGapPenalty = extendGapPenalty;
        this.contigSequence = os;
        this.referenceSequence = osr;
        this.alignFrame = af;
        this.median = m;
        this.openSequenceIndex = i;
    }

    public OpenSequenceAlignment jAlignerAlign() throws OutOfMemoryError, SequenceParserException, MatrixLoaderException, NegativeArraySizeException {

        int start = median - contigSequence.getLength();
        int end = median + contigSequence.getLength();
        if (start < 0) {
            start = 0;
        }
        if (end > referenceSequence.getLength()) {
            end = referenceSequence.getLength();
        }

        //Jaligner
        jaligner.Sequence s1 = SequenceParser.parse(referenceSequence.getSeqStr(start, end));
        jaligner.Sequence s2 = SequenceParser.parse(contigSequence.getSeqStr());
        Matrix matrix = MatrixLoader.load("IDENTITY");
        jaligner.Alignment alignment = SmithWatermanGotoh.align(s1, s2, matrix, openGapPenalty, extendGapPenalty);

        OpenSequenceAlignment osa = new OpenSequenceAlignment();
        osa.setStartPosRef(alignment.getStart1() + start);
        osa.setStartPosCont(alignment.getStart2());
        osa.setRefSeq(new Sequence(alignment.getSequence1()));
        osa.setContSeq(new Sequence(alignment.getSequence2()));
        osa.setMarkupSeq(new Sequence(alignment.getMarkupLine()));
        osa.setIdentity(alignment.getIdentity());
        osa.setSimilarity(alignment.getSimilarity());
        osa.setGaps(alignment.getGaps());
        osa.setScore(alignment.getScore());
        osa.setMedian(median);
        return osa;
    }

    @Override
    public void run() {
        try {
            this.alignFrame.setOpenSequenceAlignment(jAlignerAlign());
        } catch (OutOfMemoryError ex) {
            this.alignFrame.addResult(openSequenceIndex + " - " + JFinisher.textResource.getString("Error.memory"));
        } catch (NegativeArraySizeException ex) {
            this.alignFrame.addResult(openSequenceIndex + " - " + JFinisher.textResource.getString("Error.largeContig"));
        } catch (Exception ex) {
            this.alignFrame.addResult(openSequenceIndex + " - " + JFinisher.textResource.getString("Error.unknown"));
        }
    }
}
