package br.ufpr.tads.jfinisher.align;

import br.ufpr.tads.jfinisher.frame.AlignFrame;
import br.ufpr.bioinfo.contigsort.ContigClassify;
import br.ufpr.bioinfo.contigsort.Result;
import br.ufpr.bioinfo.sequence.fasta.Fasta;
import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.OpenSequence;
import br.ufpr.tads.jfinisher.Sequence;
import java.util.ArrayList;
import java.util.List;

public class AlignerJCS extends Thread {

    private List<OpenSequence> contigOpenSequence = new ArrayList<OpenSequence>();
    private Sequence referenceSequence;
    private int seedLength;
    private AlignFrame alignFrame;

    public AlignerJCS(int seedLength, List<OpenSequence> os, Sequence osr, AlignFrame af) {
        this.seedLength = seedLength;
        this.contigOpenSequence = os;
        this.referenceSequence = osr;
        this.alignFrame = af;
    }

    public AlignerJCS(int seedLength, OpenSequence os, Sequence osr, AlignFrame af) {
        this.seedLength = seedLength;
        this.contigOpenSequence.add(os);
        this.referenceSequence = osr;
        this.alignFrame = af;
    }

    @Override
    public void run() {
        try {
            this.alignFrame.setMedians(jContigSortAlign());
        } catch (Throwable t) {
            this.alignFrame.addResult(JFinisher.textResource.getString("Error.memory"));
        }

    }

    public List<Integer> jContigSortAlign() {
        //Jcontigsort
        List<Result> result = new ArrayList<Result>();
        List<Fasta> contigs = new ArrayList<Fasta>();
        List<Integer> medians = new ArrayList<Integer>();

        for (OpenSequence s : contigOpenSequence) {
            Fasta fst = new Fasta();
            fst.setSequence(s.getSequence().getSeq());
            contigs.add(fst);
        }

        br.ufpr.bioinfo.contigsort.Sequence reference = new br.ufpr.bioinfo.contigsort.Sequence(referenceSequence.getSeq());
        result = new ContigClassify().process(reference, contigs, seedLength);

        for (Result r : result) {
            int m = r.getMediana();
            medians.add(m);
        }
        return medians;
    }
}
