package br.ufpr.tads.jfinisher.frame;

import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.action.ExportAction;
import br.ufpr.tads.jfinisher.action.LoadProjectAction;
import br.ufpr.tads.jfinisher.action.NewProjectAction;
import br.ufpr.tads.jfinisher.action.OpenContigsAction;
import br.ufpr.tads.jfinisher.action.OpenReferenceAction;
import br.ufpr.tads.jfinisher.action.SaveProjectAction;
import br.ufpr.tads.jfinisher.enumerator.ExportType;
import br.ufpr.tads.jfinisher.enumerator.PanelType;
import br.ufpr.tads.jfinisher.sequencepanel.SequenceScrollPaneContig;
import br.ufpr.tads.jfinisher.sequencepanel.SequenceScrollPaneMap;
import br.ufpr.tads.jfinisher.sequencepanel.SequenceScrollPaneReference;
import br.ufpr.tads.jfinisher.util.WindowUtil;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

public class MainFrame extends javax.swing.JFrame {

    public SequenceScrollPaneReference sspReference;
    public SequenceScrollPaneContig sspContigs;
    public SequenceScrollPaneMap sspMap;

    public MainFrame() {

        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnsupportedLookAndFeelException ex) {
            Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
        }


        sspReference = new SequenceScrollPaneReference();
        sspContigs = new SequenceScrollPaneContig();
        sspMap = new SequenceScrollPaneMap();

        WindowUtil.init();
        initComponents();

        sspReference.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        sspReference.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        sspReference.getHorizontalScrollBar().setUnitIncrement(br.ufpr.tads.jfinisher.JFinisher.scrollUnitIncrement);

        sspContigs.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        sspContigs.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        sspContigs.getHorizontalScrollBar().setUnitIncrement(br.ufpr.tads.jfinisher.JFinisher.scrollUnitIncrement);

        sspMap.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        sspMap.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        sspMap.getHorizontalScrollBar().setUnitIncrement(br.ufpr.tads.jfinisher.JFinisher.scrollUnitIncrement);


        JSplitPane jSlipt = new JSplitPane(JSplitPane.VERTICAL_SPLIT, sspContigs, sspMap);
        jSlipt.setDividerLocation(270);
        this.generalPanelContigsMap.add(sspReference);
        this.generalPanelReference.add(jSlipt);
        lockPanels();

    }

    //Trava barras de rolagem
    private void lockPanels() {
        //Adiciona controle para rolar barra do mapa na mesma proporo das sequencias
        sspReference.getHorizontalScrollBar().addAdjustmentListener(new AdjustmentListener() {

            @Override
            public void adjustmentValueChanged(AdjustmentEvent e) {
                controlPanels(e, PanelType.REFERENCE_PANEL);
            }
        });
        sspMap.getHorizontalScrollBar().addAdjustmentListener(new AdjustmentListener() {

            @Override
            public void adjustmentValueChanged(AdjustmentEvent e) {
                controlPanels(e, PanelType.MAP_PANEL);
            }
        });
        //Define barra da referencia como modelo da barra para os contigs
        sspContigs.getHorizontalScrollBar().setModel(sspReference.getHorizontalScrollBar().getModel());
        //Adiciona espaador para painel dos contigs ter o mesmo tamanho do painel da referencia
        sspContigs.addHorizontalSpace();
        //Move posicao das telas para o inicio
        sspReference.getHorizontalScrollBar().setValue(0);
    }

    //Destrava as barras de rolagem
    private void unlockPanels() {
        sspReference.getHorizontalScrollBar().removeAdjustmentListener(sspReference.getHorizontalScrollBar().getAdjustmentListeners()[0]);
        sspMap.getHorizontalScrollBar().removeAdjustmentListener(sspMap.getHorizontalScrollBar().getAdjustmentListeners()[0]);
        sspContigs.getHorizontalScrollBar().setModel(new JScrollBar().getModel());
        sspContigs.removeHorizontalSpace();
        sspReference.getHorizontalScrollBar().setValue(0);
        sspContigs.getHorizontalScrollBar().setValue(0);
        sspMap.getHorizontalScrollBar().setValue(0);
    }

    private void controlPanels(AdjustmentEvent e, PanelType p) {
        //So faz caso possua zoom (barras)
        if (sspMap.getZoom() > 1) {
            long mapMax = JFinisher.mainFrame.sspMap.getHorizontalScrollBar().getMaximum() - sspMap.getHorizontalScrollBar().getVisibleAmount();;
            long refMax = JFinisher.mainFrame.sspReference.getHorizontalScrollBar().getMaximum() - sspReference.getHorizontalScrollBar().getVisibleAmount();;
            long current = 0, newPosition = 0;
            if (p == PanelType.MAP_PANEL) {
                current = JFinisher.mainFrame.sspMap.getHorizontalScrollBar().getValue();
            } else if (p == PanelType.REFERENCE_PANEL) {
                current = JFinisher.mainFrame.sspReference.getHorizontalScrollBar().getValue();
            }

            if (current == 0) {
                newPosition = 0;
            } else {
                if (p == PanelType.MAP_PANEL) {
                    newPosition = (refMax * current) / mapMax;
                } else if (p == PanelType.REFERENCE_PANEL) {
                    newPosition = (mapMax * current) / refMax;
                }
            }

            //Remove listener da barra que ser movida temporariamente para nao dar loop
            AdjustmentListener alTemp;
            if (p == PanelType.MAP_PANEL) {
                alTemp = sspReference.getHorizontalScrollBar().getAdjustmentListeners()[0];
                sspReference.getHorizontalScrollBar().removeAdjustmentListener(alTemp);
                JFinisher.mainFrame.sspReference.getHorizontalScrollBar().setValue((int) newPosition);
                sspReference.getHorizontalScrollBar().addAdjustmentListener(alTemp);
            } else if (p == PanelType.REFERENCE_PANEL) {
                alTemp = sspMap.getHorizontalScrollBar().getAdjustmentListeners()[0];
                sspMap.getHorizontalScrollBar().removeAdjustmentListener(alTemp);
                JFinisher.mainFrame.sspMap.getHorizontalScrollBar().setValue((int) newPosition);
                sspMap.getHorizontalScrollBar().addAdjustmentListener(alTemp);
            }
        }

    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        generalPanelReference = new javax.swing.JPanel();
        generalPanelContigsMap = new javax.swing.JPanel();
        zoomSpinner = new javax.swing.JSpinner();
        jLabel1 = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        openReferenceMenuItem = new javax.swing.JMenuItem();
        openContigMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        newProjectMenuItem = new javax.swing.JMenuItem();
        saveProjectMenuItem = new javax.swing.JMenuItem();
        saveProjectAsMenuItem = new javax.swing.JMenuItem();
        loadProjectMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JPopupMenu.Separator();
        exitMenuItem = new javax.swing.JMenuItem();
        exportMenu = new javax.swing.JMenu();
        exportMapImagemMenuItem = new javax.swing.JMenuItem();
        exportConsensusSequenceMenuItem = new javax.swing.JMenuItem();
        exportSequencesMenuItem = new javax.swing.JMenuItem();
        toolsMenu = new javax.swing.JMenu();
        lockScrollMenuItem = new javax.swing.JCheckBoxMenuItem();
        jSeparator3 = new javax.swing.JPopupMenu.Separator();
        alignAllSequencesMenuItem = new javax.swing.JMenuItem();
        joinSequencesMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        generalPanelReference.setLayout(new java.awt.GridLayout(1, 0));

        generalPanelContigsMap.setLayout(new java.awt.GridLayout(1, 0));

        zoomSpinner.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(1), Integer.valueOf(1), null, Integer.valueOf(1)));
        zoomSpinner.setEnabled(false);
        zoomSpinner.setValue(1);
        zoomSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                zoomSpinnerStateChanged(evt);
            }
        });

        jLabel1.setText("Zoom");

        fileMenu.setMnemonic('f');
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("br/ufpr/tads/jfinisher/resource/messages"); // NOI18N
        fileMenu.setText(bundle.getString("MainFrame.fileMenu.text")); // NOI18N

        openReferenceMenuItem.setAction(new OpenReferenceAction());
        openReferenceMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        openReferenceMenuItem.setMnemonic('o');
        openReferenceMenuItem.setText(bundle.getString("MainFrame.openReferenceMenuItem.text")); // NOI18N
        fileMenu.add(openReferenceMenuItem);

        openContigMenuItem.setAction(new OpenContigsAction());
        openContigMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        openContigMenuItem.setText(bundle.getString("MainFrame.openContigsMenuItem.text")); // NOI18N
        openContigMenuItem.setEnabled(false);
        fileMenu.add(openContigMenuItem);
        fileMenu.add(jSeparator1);

        newProjectMenuItem.setAction(new NewProjectAction());
        newProjectMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        newProjectMenuItem.setText(bundle.getString("MainFrame.newProjectMenuItem.text")); // NOI18N
        newProjectMenuItem.setEnabled(false);
        fileMenu.add(newProjectMenuItem);

        saveProjectMenuItem.setAction(new SaveProjectAction(false));
        saveProjectMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        saveProjectMenuItem.setText(bundle.getString("MainFrame.saveProjectMenuItem.text")); // NOI18N
        saveProjectMenuItem.setEnabled(false);
        fileMenu.add(saveProjectMenuItem);

        saveProjectAsMenuItem.setAction(new SaveProjectAction(true));
        saveProjectAsMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        saveProjectAsMenuItem.setText(bundle.getString("MainFrame.saveProjectAsMenuItem.text")); // NOI18N
        saveProjectAsMenuItem.setEnabled(false);
        fileMenu.add(saveProjectAsMenuItem);

        loadProjectMenuItem.setAction(new LoadProjectAction());
        loadProjectMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_L, java.awt.event.InputEvent.CTRL_MASK));
        loadProjectMenuItem.setText(bundle.getString("MainFrame.loadProjectMenuItem.text")); // NOI18N
        fileMenu.add(loadProjectMenuItem);
        fileMenu.add(jSeparator2);

        exitMenuItem.setMnemonic('x');
        exitMenuItem.setText(bundle.getString("MainFrame.exitMenuItem.text")); // NOI18N
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        exportMenu.setText(bundle.getString("MainFrame.exportMenu.text")); // NOI18N

        exportMapImagemMenuItem.setAction(new ExportAction(this,ExportType.MAP));
        exportMapImagemMenuItem.setText(bundle.getString("MainFrame.exportMapImagemMenuItem.text")); // NOI18N
        exportMapImagemMenuItem.setEnabled(false);
        exportMenu.add(exportMapImagemMenuItem);

        exportConsensusSequenceMenuItem.setAction(new ExportAction(this,ExportType.CONSENSUS));
        exportConsensusSequenceMenuItem.setText(bundle.getString("MainFrame.exportConsensusSequenceMenuItem.text")); // NOI18N
        exportConsensusSequenceMenuItem.setEnabled(false);
        exportMenu.add(exportConsensusSequenceMenuItem);

        exportSequencesMenuItem.setAction(new ExportAction(this,ExportType.SEQUENCES));
        exportSequencesMenuItem.setText(bundle.getString("MainFrame.exportSequencesMenuItem.text")); // NOI18N
        exportSequencesMenuItem.setEnabled(false);
        exportMenu.add(exportSequencesMenuItem);

        menuBar.add(exportMenu);

        toolsMenu.setMnemonic('e');
        toolsMenu.setText(bundle.getString("MainFrame.toolsMenu.text")); // NOI18N

        lockScrollMenuItem.setForeground(new java.awt.Color(1, 1, 1));
        lockScrollMenuItem.setSelected(true);
        lockScrollMenuItem.setText(bundle.getString("MainFrame.lockScrollMenuItem.text")); // NOI18N
        lockScrollMenuItem.setEnabled(false);
        lockScrollMenuItem.setFocusPainted(true);
        lockScrollMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/br/ufpr/tads/jfinisher/resource/chain.png"))); // NOI18N
        lockScrollMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lockScrollMenuItemActionPerformed(evt);
            }
        });
        toolsMenu.add(lockScrollMenuItem);
        toolsMenu.add(jSeparator3);

        alignAllSequencesMenuItem.setText(bundle.getString("MainFrame.alignAllSequencesMenuItem.text")); // NOI18N
        alignAllSequencesMenuItem.setEnabled(false);
        alignAllSequencesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alignAllSequencesMenuItemActionPerformed(evt);
            }
        });
        toolsMenu.add(alignAllSequencesMenuItem);

        joinSequencesMenuItem.setText(bundle.getString("MainFrame.joinSequencesMenuItem.text")); // NOI18N
        joinSequencesMenuItem.setEnabled(false);
        joinSequencesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                joinSequencesMenuItemActionPerformed(evt);
            }
        });
        toolsMenu.add(joinSequencesMenuItem);

        menuBar.add(toolsMenu);

        helpMenu.setMnemonic('h');
        helpMenu.setText(bundle.getString("MainFrame.helpMenu.text")); // NOI18N

        aboutMenuItem.setMnemonic('c');
        aboutMenuItem.setText(bundle.getString("MainFrame.aboutMenuItem.text")); // NOI18N
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setJMenuBar(menuBar);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(471, Short.MAX_VALUE)
                .addComponent(jLabel1)
                .addGap(4, 4, 4)
                .addComponent(zoomSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 51, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addComponent(generalPanelContigsMap, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 562, Short.MAX_VALUE)
            .addComponent(generalPanelReference, javax.swing.GroupLayout.DEFAULT_SIZE, 562, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(generalPanelContigsMap, javax.swing.GroupLayout.PREFERRED_SIZE, 79, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(generalPanelReference, javax.swing.GroupLayout.DEFAULT_SIZE, 264, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(zoomSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1)))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        System.exit(0);
    }//GEN-LAST:event_exitMenuItemActionPerformed

    private void lockScrollMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lockScrollMenuItemActionPerformed
        if (lockScrollMenuItem.isSelected()) {
            lockPanels();
        } else {
            unlockPanels();
        }
    }//GEN-LAST:event_lockScrollMenuItemActionPerformed

    private void alignAllSequencesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alignAllSequencesMenuItemActionPerformed
        AlignFrame alignF = new AlignFrame(JFinisher.getContigsOpenSequence());
        alignF.setLocationRelativeTo(null);
        alignF.setVisible(true);
    }//GEN-LAST:event_alignAllSequencesMenuItemActionPerformed

private void zoomSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_zoomSpinnerStateChanged
    int val = Integer.valueOf(zoomSpinner.getValue().toString());
    sspMap.setZoom(val);
}//GEN-LAST:event_zoomSpinnerStateChanged

    private void joinSequencesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_joinSequencesMenuItemActionPerformed
        JoinSequencesFrame joinSequencesFrame = new JoinSequencesFrame();
        joinSequencesFrame.setLocationRelativeTo(null);
        joinSequencesFrame.setVisible(true);
    }//GEN-LAST:event_joinSequencesMenuItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        AboutFrame aboutFrame = new AboutFrame();
        aboutFrame.setLocationRelativeTo(null);
        aboutFrame.setVisible(true);
    }//GEN-LAST:event_aboutMenuItemActionPerformed
    /**
     * @param args the command line arguments
     */
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JMenuItem alignAllSequencesMenuItem;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenuItem exportConsensusSequenceMenuItem;
    private javax.swing.JMenuItem exportMapImagemMenuItem;
    private javax.swing.JMenu exportMenu;
    private javax.swing.JMenuItem exportSequencesMenuItem;
    private javax.swing.JMenu fileMenu;
    public javax.swing.JPanel generalPanelContigsMap;
    private javax.swing.JPanel generalPanelReference;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator2;
    private javax.swing.JPopupMenu.Separator jSeparator3;
    private javax.swing.JMenuItem joinSequencesMenuItem;
    private javax.swing.JMenuItem loadProjectMenuItem;
    private javax.swing.JCheckBoxMenuItem lockScrollMenuItem;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem newProjectMenuItem;
    private javax.swing.JMenuItem openContigMenuItem;
    private javax.swing.JMenuItem openReferenceMenuItem;
    private javax.swing.JMenuItem saveProjectAsMenuItem;
    private javax.swing.JMenuItem saveProjectMenuItem;
    private javax.swing.JMenu toolsMenu;
    private javax.swing.JSpinner zoomSpinner;
    // End of variables declaration//GEN-END:variables

    public void posOpenContig() {
        sspReference.refreshConsensusSequence(JFinisher.getContigsOpenSequence());
        posOpen();
    }

    public void posOpenReference() {
        sspReference.createConsensusSequence();
        enableFields(true);
        posOpen();
    }

    public void posOpen() {
        //Redimensiona cabealho da referncia (caso aumento o tamanho dos numeros - dezena, centena)
        sspReference.refreshRowHeaderWidth();
        sspContigs.refreshRowHeaderWidth();

        //Atualiza telas
        JFinisher.mainFrame.refreshComponents();

    }

    public void clear() {
        zoomSpinner.setValue(1);
        sspReference.clear();
        sspContigs.clear();
        sspMap.clear();
        JFinisher.clearOpenSequences();
        posOpen();
    }

    public void enableFields(boolean opt) {
        openContigMenuItem.setEnabled(opt);
        openReferenceMenuItem.setEnabled(!opt);
        newProjectMenuItem.setEnabled(opt);
        saveProjectMenuItem.setEnabled(opt);
        saveProjectAsMenuItem.setEnabled(opt);

        exportConsensusSequenceMenuItem.setEnabled(opt);
        exportMapImagemMenuItem.setEnabled(opt);
        exportSequencesMenuItem.setEnabled(opt);

        lockScrollMenuItem.setEnabled(opt);
        alignAllSequencesMenuItem.setEnabled(opt);
        joinSequencesMenuItem.setEnabled(opt);

        zoomSpinner.setEnabled(opt);
    }

    public void refreshComponents() {
        JFinisher.mainFrame.sspReference.refreshViewportView();
        JFinisher.mainFrame.sspContigs.refreshViewportView();
        JFinisher.mainFrame.sspMap.refreshViewportView();
    }
}
