package br.ufpr.tads.jfinisher.listener;

import br.ufpr.tads.jfinisher.frame.AlignFrame;
import br.ufpr.tads.jfinisher.frame.GoToFrame;
import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.OpenSequence;
import br.ufpr.tads.jfinisher.OpenSequenceAlignment;
import br.ufpr.tads.jfinisher.action.GroupCharactersAction;
import br.ufpr.tads.jfinisher.action.ShowAlignmentAction;
import br.ufpr.tads.jfinisher.action.InsertSequenceAction;
import br.ufpr.tads.jfinisher.action.RemoveSelectedSequenceAction;
import br.ufpr.tads.jfinisher.action.RemoveAlignmentAction;
import br.ufpr.tads.jfinisher.action.RemoveCharactersAction;
import br.ufpr.tads.jfinisher.action.RemoveContigAction;
import br.ufpr.tads.jfinisher.enumerator.PanelType;
import java.awt.GradientPaint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.security.acl.Group;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.SwingUtilities;

public class SequenceMenuListener extends MouseAdapter {

    private OpenSequence openSequence;
    private OpenSequenceAlignment openSequenceAlignment;
    private PanelType panelType;
    //Menu principal
    private JPopupMenu mainMenu = new JPopupMenu();
    //Remover contig
    private JMenuItem removeMenuItem;
    //Remover alinhamento
    private JMenuItem removeAlignmentMenuItem;
    //Sub menu alinhamento
    private JMenu alignmentSubMenu = new JMenu(JFinisher.textResource.getString("ContigMenuListener.alignment.text"));
    //Novo alinhamento
    private JMenuItem newAlignmentMenuItem = new JMenuItem();
    //Alinhamentos
    private JMenuItem alignmentMenuItem;
    //Separador
    private JSeparator separator = new JSeparator();
    //Remover selecao
    private JMenuItem deleteSelectedSequence;
    //Adiciona sequencia
    private JMenuItem insertSequence;
    //Copiar
    private JMenuItem copySequence;
    //Ir para
    private JMenuItem goToNavigator;
    //Agrupar caracteres N
    private JMenuItem groupChar;
    //remover caractes N
    private JMenuItem removeChar;
    //Auxiliares
    private int aux;

    //Menu padro (constroi ao incluir sequencia)
    public SequenceMenuListener(OpenSequence os, PanelType pt) {
        this.openSequence = os;
        this.panelType = pt;
        defaultMenu();
    }

    public SequenceMenuListener(OpenSequence os, OpenSequenceAlignment osa, PanelType pt) {
        this.openSequence = os;
        this.openSequenceAlignment = osa;
        this.panelType = pt;
        defaultMenu();
    }

    @Override
    public void mousePressed(MouseEvent ev) {
        if (SwingUtilities.isRightMouseButton(ev)) {


            //Remover alinhamento
            if (panelType == PanelType.MAP_PANEL) {
                if (openSequenceAlignment != null) {
                    if(removeAlignmentMenuItem == null){
                        removeAlignmentMenuItem = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.removeAlignment.text"));
                        removeAlignmentMenuItem.addActionListener(new RemoveAlignmentAction(openSequence, openSequenceAlignment));
                        mainMenu.add(removeAlignmentMenuItem, 0);
                    }
                } else if (removeAlignmentMenuItem != null) {
                    mainMenu.remove(removeAlignmentMenuItem);
                    removeAlignmentMenuItem = null;
                }
            }

            //Lista alinhamentos da sequencia
            if (panelType != PanelType.REFERENCE_PANEL) {
                if (openSequence.getAlignment().size() > 0) {
                    mainMenu.remove(alignmentSubMenu);
                    alignmentSubMenu = new JMenu(JFinisher.textResource.getString("ContigMenuListener.alignment.text"));
                    for (aux = 0; aux < openSequence.getAlignment().size(); aux++) {
                        alignmentMenuItem = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.showAlign.text") + " " + (int) (aux + 1));
                        alignmentMenuItem.addActionListener(new ShowAlignmentAction(openSequence, openSequence.getAlignment().get(aux)));
                        alignmentSubMenu.add(alignmentMenuItem);
                    }
                    mainMenu.add(alignmentSubMenu, 1);
                    mainMenu.add(separator, 2);
                }
            }


            if (panelType == PanelType.CONTIGS_PANEL) {
                //Remover sequencia selecionada
                if (openSequence.getSelectedSeq().getStart() > -1 && openSequence.getSelectedSeq().getSeq().getLength() < openSequence.getSequence().getLength()) {
                    if (deleteSelectedSequence == null) {
                        deleteSelectedSequence = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.deleteSelectedSequence.text"));
                        deleteSelectedSequence.addActionListener(new RemoveSelectedSequenceAction(openSequence));
                        mainMenu.add(deleteSelectedSequence);
                    }
                } else if (deleteSelectedSequence != null) {
                    mainMenu.remove(deleteSelectedSequence);
                    deleteSelectedSequence = null;
                }

                //Inserir sequencia
                if (openSequence.getSelectedSeq().getStart() > -1) {
                    if (insertSequence == null) {
                        insertSequence = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.insertSequence.text"));
                        insertSequence.addActionListener(new InsertSequenceAction(openSequence));
                        mainMenu.add(insertSequence);
                    }
                } else if (insertSequence != null) {
                    mainMenu.remove(insertSequence);
                    insertSequence = null;
                }
            }

            //Copiar sequencia
            if (panelType != PanelType.MAP_PANEL) {
                if (openSequence.getSelectedSeq().getStart() > -1) {
                    if (copySequence == null) {
                        copySequence = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.copy.text"));
                        copySequence.addActionListener(new ActionListener() {

                            @Override
                            public void actionPerformed(ActionEvent e) {
                                JFinisher.textTransfer.setClipboardContents(openSequence.getSelectedSeq().getSeq().getSeqStr());
                            }
                        });
                        mainMenu.add(copySequence);
                    }
                } else if (copySequence != null) {
                    mainMenu.remove(copySequence);
                    copySequence = null;
                }
            }

            //Mostra menu
            mainMenu.show(ev.getComponent(), ev.getX(), ev.getY());
        }

    }

    private void defaultMenu() {
        if (panelType != PanelType.REFERENCE_PANEL) {
            newAlignmentMenuItem = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.alignContig.text"));
            newAlignmentMenuItem.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    AlignFrame alignF = new AlignFrame(openSequence);
                    alignF.setLocationRelativeTo(null);
                    alignF.setVisible(true);
                }
            });
            mainMenu.add(newAlignmentMenuItem);

            //Remover contig
            removeMenuItem = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.removeContig.text"));
            removeMenuItem.addActionListener(new RemoveContigAction(openSequence));
            mainMenu.add(removeMenuItem);
        }
        
        if(panelType!=PanelType.MAP_PANEL){
            goToNavigator = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.goto.text"));
            goToNavigator.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    GoToFrame goToFrame = new GoToFrame(openSequence, panelType);
                    goToFrame.setLocationRelativeTo(null);
                    goToFrame.setVisible(true);
                }
            });
            mainMenu.add(goToNavigator);
        }
        
        if(panelType==PanelType.CONTIGS_PANEL){
            groupChar = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.groupChar.text"));
            groupChar.addActionListener(new GroupCharactersAction(openSequence, "N"));
            mainMenu.add(groupChar);
            
            removeChar = new JMenuItem(JFinisher.textResource.getString("ContigMenuListener.removeChar.text"));
            removeChar.addActionListener(new RemoveCharactersAction(openSequence, "N"));
            mainMenu.add(removeChar);
        }
    }
}
