/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package br.ufpr.tads.jfinisher.sequencepanel;

import br.ufpr.bioinfo.genbank.Entry;
import br.ufpr.tads.jfinisher.listener.SequenceMenuListener;
import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.OpenSequence;
import br.ufpr.tads.jfinisher.Sequence;
import br.ufpr.tads.jfinisher.enumerator.PanelType;
import br.ufpr.tads.jfinisher.listener.SequenceMouseListener;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;
import javax.swing.ToolTipManager;

/**
 *
 * @author vitor
 */
public class SequencePanelSeq extends SequencePanel {

    private OpenSequence openSequence;
    int startX;
    int startY = 0;
    int width;
    int height = JFinisher.fontHeight;
    private List<Rectangle> gbkCdsEntry;
    private List<Rectangle> gbkGeneEntry;
    private List<Rectangle> nEntry;

    @Override
    public boolean contains(int x, int y) {
        int position = (int) Math.ceil(x / JFinisher.fontWidht) + 1;
        if(openSequence.getOpenSequenceAlignmentSelected()!=null){
            this.setToolTipText(String.valueOf(position) + " / " + String.valueOf(position + openSequence.getOpenSequenceAlignmentSelected().getPosition()));
        }else{
            this.setToolTipText(String.valueOf(position));
        }
        return super.contains(x, y);
    }

    public SequencePanelSeq(OpenSequence os) {
        //Mostra tooltip imediatamente
        ToolTipManager.sharedInstance().setInitialDelay(0);
        
        this.openSequence = os;

        SequenceMouseListener sml;
        if (!openSequence.isReferenceSequence()) {
            this.addMouseListener(new SequenceMenuListener(openSequence, PanelType.CONTIGS_PANEL));
            sml = new SequenceMouseListener(openSequence, PanelType.CONTIGS_PANEL);
            this.addMouseListener(sml);
            this.addMouseMotionListener(sml);
        } else {
            this.addMouseListener(new SequenceMenuListener(os, PanelType.REFERENCE_PANEL));
            sml = new SequenceMouseListener(openSequence, PanelType.REFERENCE_PANEL);
            this.addMouseListener(sml);
            this.addMouseMotionListener(sml);
        }

        packPanel(openSequence.getSequence());

    }

    @Override
    protected void paintComponent(Graphics g) {
        //Grifa texto
        if (openSequence.getSelectedSeq().getStart() != -1 && openSequence.getSelectedSeq().getEnd() != -1) {

            super.paintComponent(g);

            startX = openSequence.getSelectedSeq().getStart() * JFinisher.fontWidht;
            width = JFinisher.fontWidht * openSequence.getSelectedSeq().getSeq().getLength();
            //Borda
            g.setColor(Color.black);
            g.drawRect(startX - 1, startY, width + 2, height - 1);
            //Retangulo
            g.setColor(Color.yellow);
            g.fillRect(startX, startY + 1, width + 1, height - 2);

        }
        
        //Cria retangulos das entradas Gbk (apenas na primeira vez)
        if (openSequence.getGbkEntry() != null && openSequence.isReferenceSequence()) {
            if (gbkCdsEntry == null) {
                this.gbkCdsEntry = createGbkCdsEntry();
            }
            if (gbkGeneEntry == null) {
                this.gbkGeneEntry = createGbkGeneEntry();
            }
        }
        
        //Desenha retangulos
        Graphics2D g2 = (Graphics2D) g;
        if (gbkCdsEntry != null) {
            for (Rectangle rct : gbkCdsEntry) {
                g2.setColor(Color.red);
                g2.draw(rct);
            }
        }
        if (gbkGeneEntry != null) {
            for (Rectangle rct : gbkGeneEntry) {
                g2.setColor(Color.blue);
                g2.draw(rct);
            }
        }

        //Marca N's
        if (nEntry == null) {
            nEntry = createNEntry();
        } else {
            for (Rectangle rct : nEntry) {
                g2.setColor(Color.white);
                g2.fill(rct);
            }
        }

    }

    public void highlight() {
        this.repaint();
    }

    public void clearHighlight() {
        openSequence.getSelectedSeq().clear();
        this.repaint();
    }

    public void refreshSequence() {
        nEntry = null;
        packPanel(openSequence.getSequence());
        this.repaint();
        this.revalidate();
    }

    private List<Rectangle> createGbkCdsEntry() {
        List<Rectangle> rect = new ArrayList<Rectangle>();
        Entry e = openSequence.getGbkEntry();
        for (int i = 0; i < e.getFeatures().size(); i++) {
            if ("CDS".equalsIgnoreCase(e.getFeatures().get(i).getKey())) {
                startX = e.getFeatures().get(i).getLocation().getMinor() * JFinisher.fontWidht;
                width = (e.getFeatures().get(i).getLocation().getMajor() - e.getFeatures().get(i).getLocation().getMinor()) * JFinisher.fontWidht;
                rect.add(new Rectangle(startX, startY + 1, width + 1, height - 2));
            }
        }
        return rect;
    }

    private List<Rectangle> createGbkGeneEntry() {
        List<Rectangle> rect = new ArrayList<Rectangle>();
        Entry e = openSequence.getGbkEntry();
        for (int i = 0; i < e.getFeatures().size(); i++) {
            if ("gene".equalsIgnoreCase(e.getFeatures().get(i).getKey())) {
                startX = e.getFeatures().get(i).getLocation().getMinor() * JFinisher.fontWidht;
                width = (e.getFeatures().get(i).getLocation().getMajor() - e.getFeatures().get(i).getLocation().getMinor()) * JFinisher.fontWidht;
                rect.add(new Rectangle(startX + 1, startY +2, width - 1, height - 4));
            }
        }
        return rect;
    }

    private List<Rectangle> createNEntry() {
        List<Rectangle> rect = new ArrayList<Rectangle>();
        for (int i = 0; i <= openSequence.getSequence().getLength() - 1; i++) {
            if (openSequence.getSequence().getSeqStr(i, i).equalsIgnoreCase("n")) {
                startX = i * JFinisher.fontWidht;
                width = JFinisher.fontWidht;
                rect.add(new Rectangle(startX, startY + 1, width + 1, height - 2));
            }
        }
        return rect;
    }
}
