package br.ufpr.tads.jfinisher.sequencepanel;

import br.ufpr.tads.jfinisher.JFinisher;
import br.ufpr.tads.jfinisher.OpenSequence;
import br.ufpr.tads.jfinisher.OpenSequenceAlignment;
import br.ufpr.tads.jfinisher.enumerator.PanelType;
import br.ufpr.tads.jfinisher.listener.SequenceMenuListener;
import br.ufpr.tads.jfinisher.listener.MapMouseListener;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.image.BufferedImage;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.Spring;
import javax.swing.SpringLayout;
import javax.swing.border.Border;

public class SequenceScrollPaneMap extends JScrollPane {

    private SpringLayout sLayout = new SpringLayout();
    private SpringLayout.Constraints slCons;
    private JPanel linesPanel = new JPanel(sLayout);
    private JPanel countPanel = new JPanel(sLayout);
    private Border paddingBorder = BorderFactory.createEmptyBorder(4, 4, 4, 4);
    private Border border = BorderFactory.createLineBorder(Color.BLACK);
    private Border countBorder = BorderFactory.createMatteBorder(0, 1, 0, 0, Color.BLACK);
    public static final Color referenceColor = Color.YELLOW;
    public static final Color contigColor = Color.YELLOW;
    private static Font countFont = new Font("Monospaced", 0, 10);
    private int lineHeight = 10;
    private int zoom = 1;
    private int spaceGap = 10;

    public SequenceScrollPaneMap() {
        //Ao redimensionar, redimensiona painel contador junto
        this.addComponentListener(new ComponentAdapter() {

            @Override
            public void componentResized(ComponentEvent e) {
                refreshViewportView();
                refreshCounterPanel();
            }
        });
    }

    public void addSequence(final OpenSequence os) {

        //Cria label
        JLabel sequenceLabel = createSequenceLabel(os);

        //Adiciona label a lista para referencia da sequencia de outros alinhamentos (para acesso posterior)
        os.setMapLabel(sequenceLabel);

        //Adiciona label ao painel
        linesPanel.add(os.getMapLabel());

        slCons = sLayout.getConstraints(sequenceLabel);

        //Caso seja a referencia
        if (os.isReferenceSequence()) {
            sequenceLabel.setBackground(referenceColor);

            //Posiciona label no springlayout
            slCons.setX(Spring.constant(0));
            slCons.setY(Spring.constant(0));
            slCons.setWidth(Spring.scale(sLayout.getConstraint(SpringLayout.EAST, linesPanel), 1f));

            refreshCounterPanel();
        } else {
            sequenceLabel.setBackground(contigColor);
            sequenceLabel.addMouseListener(new SequenceMenuListener(os, PanelType.MAP_PANEL));
            sequenceLabel.addMouseListener(new MapMouseListener(os));

            float width = (float) os.getSequence().getLength() / JFinisher.getReferenceOpenSequence().getSequence().getLength();
            slCons.setX(Spring.constant(0));
            slCons.setY(sLayout.getConstraint(SpringLayout.SOUTH, JFinisher.getOpenSequence(JFinisher.getOpenSequenceIndex(os) - 1).getMapLabel()));
            slCons.setWidth(Spring.scale(sLayout.getConstraint(SpringLayout.EAST, linesPanel), width));
        }
    }

    public void addSequenceAlignment(OpenSequence os, OpenSequenceAlignment osa) {

        // o primeiro alinhamento, esconde label padrao
        if (os.getAlignment().size() == 1) {
            os.getMapLabel().setVisible(false);
        }

        JLabel sequenceLabel = createSequenceLabel(os);

        //Referencia label para cada alinhamento (para remocao)
        osa.setMapLabel(sequenceLabel);

        //Adiciona label ao painel
        linesPanel.add(sequenceLabel);

        sequenceLabel.setBackground(contigColor);
        sequenceLabel.addMouseListener(new SequenceMenuListener(os, osa, PanelType.MAP_PANEL));
        sequenceLabel.addMouseListener(new MapMouseListener(os, osa));

        slCons = sLayout.getConstraints(sequenceLabel);

        //Posicao no mapa (x)
        float x = (float) osa.getPosition() / JFinisher.getReferenceOpenSequence().getSequence().getLength();
        slCons.setX(Spring.scale(sLayout.getConstraint(SpringLayout.EAST, linesPanel), x));

        //Posicao no mapa (y) em baixo ao label referencia (anterior)
        slCons.setY(sLayout.getConstraint(SpringLayout.SOUTH, JFinisher.getOpenSequence(JFinisher.getOpenSequenceIndex(os) - 1).getMapLabel()));

        //Tamanho no mapa (width)
        float width = (float) os.getSequence().getLength() / JFinisher.getReferenceOpenSequence().getSequence().getLength();
        slCons.setWidth(Spring.scale(sLayout.getConstraint(SpringLayout.EAST, linesPanel), width));

    }

    public void refreshSequence(OpenSequence os) {
        slCons = sLayout.getConstraints(os.getMapLabel());
        //Tamanho no mapa (width)
        float width = (float) os.getSequence().getLength() / JFinisher.getReferenceOpenSequence().getSequence().getLength();
        slCons.setWidth(Spring.scale(sLayout.getConstraint(SpringLayout.EAST, linesPanel), width));

        for (OpenSequenceAlignment osa : os.getAlignment()) {
            slCons = sLayout.getConstraints(osa.getMapLabel());
            //Tamanho no mapa (width)
            width = (float) os.getSequence().getLength() / JFinisher.getReferenceOpenSequence().getSequence().getLength();
            slCons.setWidth(Spring.scale(sLayout.getConstraint(SpringLayout.EAST, linesPanel), width));
        }

        refreshViewportView();
    }

    public JLabel createSequenceLabel(OpenSequence os) {
        //Cria label
        JLabel sequenceLabel = new JLabel();
        sequenceLabel.setBorder(BorderFactory.createCompoundBorder(border, paddingBorder));
        sequenceLabel.setPreferredSize(new Dimension(0, lineHeight));
        sequenceLabel.setOpaque(true);
        //sequenceLabel.setFont(new Font("Monospaced", 0, 9));
        //sequenceLabel.setText(String.valueOf(JFinisher.getOpenSequenceIndex(os)));
        sequenceLabel.setToolTipText(JFinisher.getOpenSequenceIndex(os) + " " + os.getDescription());
        return sequenceLabel;
    }

    public void refreshViewportView() {
        //Ajusta painel para scroll ficar correto
        int height = JFinisher.getOpenSequencesSize() * lineHeight;
        linesPanel.setPreferredSize(new Dimension(this.getZoom() * (getSize().width - 20), height));
        this.setColumnHeaderView(countPanel);
        this.setViewportView(linesPanel);
    }

    public int getZoom() {
        return zoom;
    }

    public void setZoom(int zoom) {
        if (zoom >= 1) {
            this.zoom = zoom;
            refreshCounterPanel();
            refreshViewportView();
        }
    }

    private void refreshCounterPanel() {

        if (!JFinisher.getOpenSequences().isEmpty()) {
            //Tamanho total da seq referencia
            int sequenceLength = JFinisher.getReferenceOpenSequence().getSequence().getLength();

            //Espao que cada gap ter na tela
            float gap = (float) sequenceLength / (spaceGap * this.getZoom());

            //So atualiza contador caso gap seja maior que 5 (limite maximo)
            if (gap >= 5) {

                //Remove todos os elementos do painel para gerar outro contador (zoom out/in)
                countPanel.removeAll();

                for (float i = 0; i <= sequenceLength; i += gap) {
                    //Posicao na sequencia de cada gap
                    int sequencePosition = (int) i;

                    //Cria label com contador
                    JLabel countLabel = new JLabel(String.valueOf(sequencePosition));
                    countLabel.setFont(countFont);
                    countLabel.setBorder(countBorder);
                    countPanel.add(countLabel);

                    //Define tamanhos label dento do springlayout
                    slCons = sLayout.getConstraints(countLabel);
                    slCons.setY(Spring.constant(0));
                    slCons.setX(Spring.scale(sLayout.getConstraint(SpringLayout.EAST, countPanel), i / sequenceLength));
                }
            }

            //Tamanho maximo do painel
            countPanel.setPreferredSize(new Dimension(this.getZoom() * (getSize().width - 20), getFontMetrics(countFont).getHeight()));
            this.setColumnHeaderView(countPanel);
        }
    }

    public void removeSequence(OpenSequence os) {

        //Remove alinhamentos
        for (OpenSequenceAlignment osa : os.getAlignment()) {
            slCons = sLayout.getConstraints(osa.getMapLabel());
            slCons.setX(Spring.constant(0));
            slCons.setY(Spring.constant(0));
            slCons.setWidth(Spring.constant(0));
        }

        //Esconde linha da sequencia
        slCons = sLayout.getConstraints(os.getMapLabel());
        slCons.setX(Spring.constant(0));
        slCons.setY(Spring.constant(0));
        slCons.setWidth(Spring.constant(0));

        //Caso no seja o primeiro (nem ultimo) e possua linha abaixo, vincula proxima linha a linha anterior
        if (JFinisher.getOpenSequenceIndex(os) > 1 && JFinisher.getOpenSequencesSize() - 1 > JFinisher.getOpenSequenceIndex(os)) {
            slCons = sLayout.getConstraints(JFinisher.getOpenSequence(JFinisher.getOpenSequenceIndex(os) + 1).getMapLabel());
            slCons.setY(sLayout.getConstraint(SpringLayout.SOUTH, JFinisher.getOpenSequence(JFinisher.getOpenSequenceIndex(os) - 1).getMapLabel()));
        }

        this.refreshViewportView();
    }

    public void removeSequenceAlignment(OpenSequence os, OpenSequenceAlignment osa) {
        //Esconde linha da sequencia
        slCons = sLayout.getConstraints(osa.getMapLabel());
        slCons.setX(Spring.constant(0));
        slCons.setY(Spring.constant(0));
        slCons.setWidth(Spring.constant(0));

        this.refreshViewportView();
    }

    public void clear() {
        this.countPanel.removeAll();
        this.linesPanel.removeAll();
    }

    public void paintAlign(OpenSequence os, OpenSequenceAlignment osa) {
        //Caso seja nulo, mostra label default
        if (osa == null) {
            os.getMapLabel().setVisible(true);
        } else {
            //Loop nos alinhamentos e so preenche o que foi solicitado, deixando os outros vazios
            for (OpenSequenceAlignment auxOsa : os.getAlignment()) {
                if (auxOsa != osa) {
                    auxOsa.getMapLabel().setOpaque(false);
                } else {
                    auxOsa.getMapLabel().setOpaque(true);
                }
            }
        }
        refreshViewportView();
    }

    public void showSequence(OpenSequence openSequence, OpenSequenceAlignment openSequenceAlignment) {
        //Armazena alinhamento anteror e seleciona novo
        OpenSequenceAlignment oldOsa = openSequence.getOpenSequenceAlignmentSelected();
        openSequence.setOpenSequenceAlignmentSelected(openSequenceAlignment);

        if (oldOsa != openSequenceAlignment) {
            //Move sequencia na tela
            JFinisher.mainFrame.sspContigs.moveSequence(openSequence, openSequenceAlignment, oldOsa);
            //Marca sequencia no mapa
            paintAlign(openSequence, openSequenceAlignment);
        }
        
        JFinisher.mainFrame.sspContigs.setViewportPosition(openSequence.getPosition(), openSequence);
        
    }

    public BufferedImage createImage() throws NegativeArraySizeException {

        JPanel imagePanel = linesPanel;
        int width = countPanel.getWidth();
        int height = countPanel.getHeight() + linesPanel.getHeight();

        BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        Graphics2D g = bi.createGraphics();
        imagePanel.paint(g);
        g.dispose();

        return bi;

    }
}
